@@ -43,6 +43,8 @@
     use constant MPFR_PV_NV_BUG         => Math::MPFR::Random::_has_pv_nv_bug();
 
     # https://github.com/StrawberryPerl/Perl-Dist-Strawberry/issues/226
+    # Math::MPFR::Random::_buggy() was modified in version 4.34 to
+    # accommodate use of vcpkg-built gmp & mpfr libraries on MS Windows.
     use constant WIN32_FMT_BUG          => Math::MPFR::Random::_buggy();
 
     use constant NV_IS_DOUBLEDOUBLE     => 1 + (2 ** -200) > 1 ? 1 : 0;
@@ -68,6 +70,8 @@
     '-='   => \&overload_sub_eq,
     '*='   => \&overload_mul_eq,
     '/='   => \&overload_div_eq,
+    '%'    => \&overload_fmod,
+    '%='   => \&overload_fmod_eq,
     '""'   => \&overload_string,
     '>'    => \&overload_gt,
     '>='   => \&overload_gte,
@@ -82,10 +86,10 @@
     'abs'  => \&overload_abs,
     '**'   => \&overload_pow,
     '**='  => \&overload_pow_eq,
-    '<<'   => \&overload_mul_2exp,
-    '<<='  => \&overload_mul_2exp_eq,
-    '>>'   => \&overload_div_2exp,
-    '>>='  => \&overload_div_2exp_eq,
+    '<<'   => \&overload_lshift,
+    '<<='  => \&overload_lshift_eq,
+    '>>'   => \&overload_rshift,
+    '>>='  => \&overload_rshift_eq,
     'atan2'=> \&overload_atan2,
     'cos'  => \&overload_cos,
     'sin'  => \&overload_sin,
@@ -116,9 +120,9 @@ Rmpfr_agm Rmpfr_ai
 Rmpfr_asin Rmpfr_asinh Rmpfr_asinpi Rmpfr_asinu
 Rmpfr_atan Rmpfr_atan2 Rmpfr_atan2pi Rmpfr_atan2u Rmpfr_atanh Rmpfr_atanpi Rmpfr_atanu
 Rmpfr_beta
-Rmpfr_buildopt_decimal_p Rmpfr_buildopt_float128_p Rmpfr_buildopt_gmpinternals_p
-Rmpfr_buildopt_sharedcache_p Rmpfr_buildopt_tls_p Rmpfr_buildopt_tune_case
-Rmpfr_can_round Rmpfr_cbrt Rmpfr_ceil Rmpfr_check_range
+Rmpfr_buildopt_decimal_p Rmpfr_buildopt_float16_p Rmpfr_buildopt_float128_p
+Rmpfr_buildopt_gmpinternals_p Rmpfr_buildopt_sharedcache_p Rmpfr_buildopt_tls_p
+Rmpfr_buildopt_tune_case Rmpfr_can_round Rmpfr_cbrt Rmpfr_ceil Rmpfr_check_range
 Rmpfr_clear Rmpfr_clear_divby0 Rmpfr_clear_erangeflag Rmpfr_clear_flags Rmpfr_clear_inexflag
 Rmpfr_clear_nanflag Rmpfr_clear_overflow Rmpfr_clear_underflow Rmpfr_clears
 Rmpfr_cmp Rmpfr_cmp_IV Rmpfr_cmp_NV Rmpfr_cmp_d Rmpfr_cmp_f Rmpfr_cmp_float128 Rmpfr_cmp_ld
@@ -145,8 +149,8 @@ Rmpfr_free_cache Rmpfr_free_cache2 Rmpfr_free_pool Rmpfr_frexp Rmpfr_gamma Rmpfr
 Rmpfr_get_DECIMAL128 Rmpfr_get_DECIMAL64 Rmpfr_get_FLOAT128 Rmpfr_get_IV Rmpfr_get_LD Rmpfr_get_NV
 Rmpfr_get_d Rmpfr_get_d1 Rmpfr_get_d_2exp Rmpfr_get_default_prec Rmpfr_get_default_rounding_mode
 Rmpfr_get_emax Rmpfr_get_emax_max Rmpfr_get_emax_min Rmpfr_get_emin Rmpfr_get_emin_max
-Rmpfr_get_emin_min Rmpfr_get_exp Rmpfr_get_f Rmpfr_get_float128 Rmpfr_get_flt Rmpfr_get_ld
-Rmpfr_get_ld_2exp Rmpfr_get_patches Rmpfr_get_prec Rmpfr_get_q Rmpfr_get_si Rmpfr_get_sj
+Rmpfr_get_emin_min Rmpfr_get_exp Rmpfr_get_f Rmpfr_get_float128 Rmpfr_get_flt Rmpfr_get_float16
+Rmpfr_get_ld Rmpfr_get_ld_2exp Rmpfr_get_patches Rmpfr_get_prec Rmpfr_get_q Rmpfr_get_si Rmpfr_get_sj
 Rmpfr_get_str Rmpfr_get_str_ndigits Rmpfr_get_str_ndigits_alt Rmpfr_get_ui Rmpfr_get_uj
 Rmpfr_get_version Rmpfr_get_z Rmpfr_get_z_2exp Rmpfr_get_z_exp
 Rmpfr_grandom Rmpfr_greater_p Rmpfr_greaterequal_p Rmpfr_hypot Rmpfr_inexflag_p Rmpfr_inf_p
@@ -175,7 +179,7 @@ Rmpfr_root Rmpfr_rootn_ui Rmpfr_round Rmpfr_round_nearest_away Rmpfr_roundeven
 Rmpfr_sec Rmpfr_sech Rmpfr_set Rmpfr_set_DECIMAL128 Rmpfr_set_DECIMAL64 Rmpfr_set_FLOAT128
 Rmpfr_set_IV Rmpfr_set_LD Rmpfr_set_NV Rmpfr_set_d Rmpfr_set_default_prec
 Rmpfr_set_default_rounding_mode Rmpfr_set_divby0 Rmpfr_set_emax Rmpfr_set_emin Rmpfr_set_erangeflag
-Rmpfr_set_exp Rmpfr_set_f Rmpfr_set_float128 Rmpfr_set_flt Rmpfr_set_inexflag Rmpfr_set_inf
+Rmpfr_set_exp Rmpfr_set_f Rmpfr_set_float128 Rmpfr_set_flt Rmpfr_set_float16 Rmpfr_set_inexflag Rmpfr_set_inf
 Rmpfr_set_ld Rmpfr_set_nan Rmpfr_set_nanflag Rmpfr_set_overflow Rmpfr_set_prec Rmpfr_set_prec_raw
 Rmpfr_set_q Rmpfr_set_si Rmpfr_set_si_2exp Rmpfr_set_sj Rmpfr_set_sj_2exp Rmpfr_set_str Rmpfr_set_ui
 Rmpfr_set_ui_2exp Rmpfr_set_uj Rmpfr_set_uj_2exp Rmpfr_set_underflow Rmpfr_set_z Rmpfr_set_z_2exp
@@ -192,13 +196,13 @@ Rmpfr_z_div Rmpfr_z_sub Rmpfr_zero_p Rmpfr_zeta Rmpfr_zeta_ui
 TRmpfr_inp_str TRmpfr_out_str
 anytoa atodouble atonum atonv
 check_exact_decimal decimalize doubletoa dragon_test
-fr_cmp_q_rounded mpfr_max_orig_len mpfr_min_inter_prec mpfrtoa numtoa nvtoa nvtoa_test
+fr_cmp_q_rounded mpfr_max_orig_len mpfr_min_inter_prec mpfrtoa numtoa nvtoa nv2mpfr nvtoa_test
 prec_cast q_add_fr q_cmp_fr q_div_fr q_mul_fr q_sub_fr rndna
 );
 
     @Math::MPFR::EXPORT_OK = (@tags, 'bytes');
 
-    our $VERSION = '4.33';
+    our $VERSION = '4.35';
     #$VERSION = eval $VERSION;
 
     Math::MPFR->DynaLoader::bootstrap($VERSION);
@@ -458,18 +462,14 @@ sub new {
 
 sub Rmpfr_printf {
     if(@_ == 3){
-      if(_itsa($_[1]) == 2) {wrap_mpfr_printf_rnd(@_)}
+      if(_itsa($_[1]) == 2) {wrap_mpfr_printf_rnd(@_)} # $_[1] is rounding argument (IOK).
       else {die "The second (of 3) arguments given to Rmpfr_printf() is not a valid rounding argument"}
     }
-    else {die "Rmpfr_printf must take 2 or 3 arguments: format string, [rounding,], and variable" if @_ != 2;
-      if(WIN32_FMT_BUG) {
-        my $revised = _rewrite_fmt_arg($_[0]);
-        if($revised) {
-          wrap_mpfr_printf($revised, _to_mpfr_object($_[1]));
-        }
-        else {
-          wrap_mpfr_printf(@_);
-        }
+    else {
+      die "Rmpfr_printf must take 2 or 3 arguments: format string, [rounding,], and variable" if @_ != 2;
+      my $revised = _rewrite_fmt_arg($_[0], $_[1]);
+      if($revised) {
+        wrap_mpfr_printf($revised, nv2mpfr($_[1]));
       }
       else {
         wrap_mpfr_printf(@_);
@@ -482,16 +482,11 @@ sub Rmpfr_fprintf {
       if(_itsa($_[2]) == 2) {wrap_mpfr_fprintf_rnd(@_)}
       else {die "The third (of 4) arguments given to Rmpfr_fprintf() is not a valid rounding argument"}
     }
-    else {die "Rmpfr_fprintf must take 3 or 4 arguments: filehandle, format string, [rounding,], and variable" if @_ != 3;
-
-      if(WIN32_FMT_BUG) {
-        my $revised = _rewrite_fmt_arg($_[1]);
-        if($revised) {
-          wrap_mpfr_fprintf($_[0], $revised, _to_mpfr_object($_[2]));
-        }
-        else {
-          wrap_mpfr_fprintf(@_);
-        }
+    else {
+      die "Rmpfr_fprintf must take 3 or 4 arguments: filehandle, format string, [rounding,], and variable" if @_ != 3;
+      my $revised = _rewrite_fmt_arg($_[1], $_[2]);
+      if($revised) {
+        wrap_mpfr_fprintf($_[0], $revised, nv2mpfr($_[2]));
       }
       else {
         wrap_mpfr_fprintf(@_);
@@ -510,16 +505,18 @@ sub Rmpfr_sprintf {
     }
     die "Rmpfr_sprintf must take 4 or 5 arguments: buffer, format string, [rounding,] variable and buffer size" if @_ != 4;
 
-    # If we've got to here then @_ should comprise of only 4 arguments.
-
-    if(WIN32_FMT_BUG) {
-      my $revised = _rewrite_fmt_arg($_[1]);
-      if($revised) {
-        $len = wrap_mpfr_sprintf($_[0], $revised, _to_mpfr_object($_[2]), $_[3]);
-        return $len;
-      }
+    my $revised = _rewrite_fmt_arg($_[1], $_[2]);
+    if($revised) {
+      $len = wrap_mpfr_sprintf($_[0], $revised, nv2mpfr($_[2]), $_[3]);
+      return $len;
     }
 
+# Alternatively, this should work, but it crashes intermittently
+#    if(!_hex_fmt_ok($_[1], $_[2])) {
+#      $len = _gmp_sprintf_nv(@_);
+#      return $len;
+#    }
+
     $len = wrap_mpfr_sprintf(@_);
     return $len;
 }
@@ -535,14 +532,10 @@ sub Rmpfr_snprintf {
     }
     die "Rmpfr_snprintf must take 5 or 6 arguments: buffer, bytes written, format string, [rounding,], variable and buffer size" if @_ != 5;
 
-    # If we've got to here then @_ should comprise of only 5 arguments.
-
-    if(WIN32_FMT_BUG) {
-      my $revised = _rewrite_fmt_arg($_[2]);
-      if($revised) {
-        $len = wrap_mpfr_snprintf($_[0], $_[1], $revised, _to_mpfr_object($_[3]), $_[4]);
-        return $len;
-      }
+    my $revised = _rewrite_fmt_arg($_[2], $_[3]);
+    if($revised) {
+      $len = wrap_mpfr_snprintf($_[0], $_[1], $revised, nv2mpfr($_[3]), $_[4]);
+      return $len;
     }
 
     $len = wrap_mpfr_snprintf(@_);
@@ -1671,39 +1664,138 @@ sub _decrement {
   return $ret;
 }
 
-sub _rewrite_fmt_arg { # Called only if the constant WIN32_FMT_BUG is 1
+sub _rewrite_fmt_arg {
+  # Can Return a true value if and only if the constant
+  # WIN32_FMT_BUG is set to a true value, in which case it
+  # returns a rewritten format string.
+  # We allow "%a"/"%A" formatting only if nvtype is 'double' and
+  # we allow "%La"/"%LA" formatting only if nvtype is 'long double'.
+
   my $arg = shift;
   # First check for a match of (eg) "%La" or "%A"
   # at the beginning of $arg.
   if($arg =~ /^%L?[a,A]/) {
-    # Should not be multiple matches.
-    $arg =~ s/^%L?/%R/;
-    return $arg;
+    my $match = $&;
+    if(!Math::MPFR::_SvNOK($_[0])) { die "\"$match\" formatting applies only to NVs. Use \"%Ra\" for Math::MPFR objects." }
+    if($match =~ /L/) { die "\"$match\" formatting applies only to long doubles." unless $Config{nvtype} eq 'long double'; }
+    else { die "\"$match\" formatting applies only to doubles." unless $Config{nvtype} eq 'double'; }
+    if(WIN32_FMT_BUG) {
+      $arg =~ s/^%L?/%R/;
+      return $arg;
+    }
+    return '';
   }
 
   # Need to match (eg) "%La" or "%A", but also to NOT match (eg) "%%La" or "%%A".
   if($arg =~ /[^%]%L?[a,A]/) {
-    # Should not be multiple matches.
-    my $and_init = $&;
-    my $temp = $&;
-    my $start = substr($temp, 0, 1, '');
-    $temp =~ s/^%L?/%R/;
-    $start .= $temp;
-    $arg =~ s/$and_init/$start/;
-    return $arg;
+    my $match = $&;
+    if(!Math::MPFR::_SvNOK($_[0])) { die "\"", substr($match, 1), "\" formatting applies only to NVs. Use \"%Ra\" for Math::MPFR objects." }
+    if($match =~ /L/) { die "\"", substr($match, 1), "\" formatting applies only to long doubles." unless $Config{nvtype} eq 'long double'; }
+    else { die "\"", substr($match, 1), "\" formatting applies only to doubles." unless $Config{nvtype} eq 'double'; }
+    if(WIN32_FMT_BUG) {
+      my $and_init = $match;
+      my $temp = $match;
+      my $start = substr($temp, 0, 1, '');
+      $temp =~ s/^%L?/%R/;
+      $start .= $temp;
+      $arg =~ s/$and_init/$start/;
+      return $arg;
+    }
   }
   return ''; # Nothing to modify - no action necessary.
 }
 
-sub _to_mpfr_object { # Called only if the constant WIN32_FMT_BUG is 1
-                      # && _rewrite_fmt_arg returned a non-empty string.
+sub nv2mpfr {
   my $prec = Rmpfr_get_default_prec();
   Rmpfr_set_default_prec($Math::MPFR::NV_properties{bits});
-  my $arg = Math::MPFR->new($_[0]);
+  my $ret = Math::MPFR->new($_[0]);
   Rmpfr_set_default_prec($prec);
-  return $arg;
+  return $ret;
 }
 
+sub _win32_formatting_ok {   # Duplicated in Random/Random.pm
+    # Return 1 if either __GMP_CC or __GMP_CFLAGS
+    # include the string '-D__USE_MINGW_ANSI_STDIO'.
+    # Else return 0.
+
+    my $cc = _gmp_cc();		# __GMP_CC
+    my $cflags = _gmp_cflags();	# __GMP_CFLAGS
+
+    return 1 if ( defined($cc)     && $cc     =~/\-D__USE_MINGW_ANSI_STDIO/ );
+    return 1 if ( defined($cflags) && $cflags =~/\-D__USE_MINGW_ANSI_STDIO/ );
+    return 0;
+}
+
+sub _hex_fmt_ok {
+  # Return 1 if no hex ("%a") formatting is requested.
+  # Return 1 if the requested hex format ("%a"/"%A"/"%La"/"%LA") is appropriate.
+  # If we haven't died or returned by now, then return 0 if WIN32_FMT_BUG is set;
+  # otherwise return 1.
+  # A return of 1 tells the caller (Rmpfr_*printf) that no other action is necessary.
+  # A return of 0 tells the caller(Rmpfr_*printf) that it needs to call _rewrite_fmt_arg().
+
+  my $fmt = shift;
+
+  if($fmt =~ /^%L?[a,A]|[^%]%L?[a,A]/) {
+     $fmt = $&;
+     if(!Math::MPFR::_SvNOK($_[0])) { die "\"%a\" formatting applies only to NVs. Use \"%Ra\" for Math::MPFR objects." }
+     if($fmt =~ /L/) { die "\"%La\" formatting applies only to long doubles." unless $Config{nvtype} eq 'long double'; }
+     else { die "\"%a\" formatting applies only to doubles." unless $Config{nvtype} eq 'double'; }
+     return 0 if WIN32_FMT_BUG; # Tells the caller to call the relevant _gmp_*printf_nv() function
+     return 1;
+  }
+  else {return 1} # No %a formatting requested. Proceed as normal.
+}
+
+sub overload_lshift {
+  if($_[2] || !_looks_like_number($_[1])) {
+    die "Math::MPFR: When overloading '<<', the argument that specifies the number of bits to be shifted must be a perl number";
+  }
+  return _overload_lshift(@_) if $_[1] >= 0;
+  return _overload_rshift($_[0], -$_[1], $_[2]);
+}
+
+sub overload_lshift_eq {
+  if($_[2] || !_looks_like_number($_[1])) {
+    die "Math::MPFR: When overloading '<<=', the argument that specifies the number of bits to be shifted must be a perl number";
+  }
+  return _overload_lshift_eq(@_) if $_[1] >= 0;
+  return _overload_rshift_eq($_[0], -$_[1], $_[2]);
+}
+
+sub overload_rshift {
+  if($_[2] || !_looks_like_number($_[1])) {
+    die "Math::MPFR: When overloading '>>', the argument that specifies the number of bits to be shifted must be a perl number";
+  }
+  return _overload_rshift(@_) if $_[1] >= 0;
+  return _overload_lshift($_[0], -$_[1], $_[2]);
+}
+
+sub overload_rshift_eq {
+  if($_[2] || !_looks_like_number($_[1])) {
+    die "Math::MPFR: When overloading '>>=', the argument that specifies the number of bits to be shifted must be a perl number";
+  }
+  return _overload_rshift_eq(@_) if $_[1] >= 0;
+  return _overload_lshift_eq($_[0], -$_[1], $_[2]);
+}
+
+sub overload_fmod {
+  if(ref($_[1]) ne 'Math::MPFR') {
+    return _overload_fmod($_[0], Math::MPFR->new($_[1]), 0) unless $_[2];
+    return _overload_fmod(Math::MPFR->new($_[1]), $_[0], 0);
+  }
+  return _overload_fmod(@_);
+}
+
+sub overload_fmod_eq {
+  if(ref($_[1]) ne 'Math::MPFR') {
+    return _overload_fmod_eq($_[0], Math::MPFR->new($_[1]), 0) unless $_[2];
+    return _overload_fmod_eq(Math::MPFR->new($_[1]), $_[0], 0);
+  }
+  return _overload_fmod_eq(@_);
+}
+
+
 1;
 
 __END__
