@@ -261,11 +261,11 @@ typedef _Decimal128 D128;
 /* Don't use NON_NUMERIC_CHAR_CHECK macro with Rmpfr_inp_str as this *
  * function requires a different condition (!ret vs ret).            */
 
-#define NON_NUMERIC_CHAR_CHECK \
+#define NON_NUMERIC_CHAR_CHECK(inputstr) \
        if(ret) { \
          nnum++; \
          if(SvIV(get_sv("Math::MPFR::NNW", 0))) \
-           warn("string used in %s contains non-numeric characters"
+           warn("string used (%s) in %s contains non-numeric characters", SvPV_nolen(inputstr)
 
 #define BITSEARCH_4 \
           if(tmp & 8) {				\
@@ -425,7 +425,7 @@ typedef _Decimal128 D128;
 
 # endif
 
-#elif defined(USE_LONG_DOUBLE)
+#elif defined(USE_LONG_DOUBLE) && REQUIRED_LDBL_MANT_DIG == 64
 #define MATH_MPFR_MAX_DIG 21
 #define NVSIZE_BITS 64
 #define MATH_MPFR_NV_MAX 0xf.fffffffffffffffp+16380L
